@@ -5,8 +5,8 @@ import aesara.tensor as at
 from aesara.compile.mode import optdb
 from aesara.graph.features import Feature
 from aesara.graph.op import compute_test_value
-from aesara.graph.opt import EquilibriumOptimizer, local_optimizer
-from aesara.graph.optdb import SequenceDB
+from aesara.graph.opt import local_optimizer
+from aesara.graph.optdb import EquilibriumDB, SequenceDB
 from aesara.tensor.extra_ops import BroadcastTo
 from aesara.tensor.random.op import RandomVariable
 from aesara.tensor.random.opt import (
@@ -17,13 +17,17 @@ from aesara.tensor.random.opt import (
 from aesara.tensor.subtensor import (
     AdvancedIncSubtensor,
     AdvancedIncSubtensor1,
+    AdvancedSubtensor,
+    AdvancedSubtensor1,
     IncSubtensor,
+    Subtensor,
 )
 from aesara.tensor.var import TensorVariable
 
 from aeppl.utils import indices_from_subtensor
 
 inc_subtensor_ops = (IncSubtensor, AdvancedIncSubtensor, AdvancedIncSubtensor1)
+subtensor_ops = (AdvancedSubtensor, AdvancedSubtensor1, Subtensor)
 
 
 class PreserveRVMappings(Feature):
@@ -58,34 +62,6 @@ class PreserveRVMappings(Feature):
             self.rv_values[new_r] = r_value_var
 
 
-class RVSinker(EquilibriumOptimizer):
-    """Sink `RandomVariable` `Op`s so that log-probabilities can be determined.
-
-    This optimizer is essentially a collection of `RandomVariable`-based rewrites
-    that are needed in order to compute log-probabilities for non-`RandomVariable`
-    graphs.
-    """
-
-    def __init__(self):
-        super().__init__(
-            [
-                local_dimshuffle_rv_lift,
-                local_subtensor_rv_lift,
-                naive_bcast_rv_lift,
-                incsubtensor_rv_replace,
-            ],
-            ignore_newtrees=False,
-            tracks_on_change_inputs=True,
-            max_use_ratio=10000,
-        )
-
-    # If we wanted to support the `.tag.value_var` approach,
-    # something like the following would be reasonable:
-    # def add_requirements(self, fgraph):
-    #     if not hasattr(fgraph, "preserve_rv_mappings"):
-    #         fgraph.attach_feature(PreserveRVMappings({}))
-
-
 @local_optimizer(inc_subtensor_ops)
 def incsubtensor_rv_replace(fgraph, node):
     r"""Replace `*IncSubtensor*` `Op`\s and their value variables for log-probability calculations.
@@ -107,15 +83,15 @@ def incsubtensor_rv_replace(fgraph, node):
     rv_map_feature = getattr(fgraph, "preserve_rv_mappings", None)
 
     if rv_map_feature is None:
-        return
+        return None  # pragma: no cover
 
     if not isinstance(node.op, inc_subtensor_ops):
-        return
+        return None  # pragma: no cover
 
     rv_var = node.inputs[0]
 
     if not (rv_var.owner and isinstance(rv_var.owner.op, RandomVariable)):
-        return
+        return None  # pragma: no cover
 
     data = node.inputs[1]
     idx = indices_from_subtensor(getattr(node.op, "idx_list", None), node.inputs[2:])
@@ -148,7 +124,7 @@ def naive_bcast_rv_lift(fgraph, node):
         and node.inputs[0].owner
         and isinstance(node.inputs[0].owner.op, RandomVariable)
     ):
-        return
+        return None  # pragma: no cover
 
     bcast_shape = node.inputs[1:]
 
@@ -158,7 +134,7 @@ def naive_bcast_rv_lift(fgraph, node):
     rv_node = rv_var.owner
 
     if hasattr(fgraph, "dont_touch_vars") and rv_var in fgraph.dont_touch_vars:
-        return
+        return None  # pragma: no cover
 
     size_lift_res = local_rv_size_lift.transform(fgraph, rv_node)
     if size_lift_res is None:
@@ -169,7 +145,15 @@ def naive_bcast_rv_lift(fgraph, node):
 
     rng, size, dtype, *dist_params = lifted_node.inputs
 
-    new_dist_params = [at.broadcast_to(param, bcast_shape) for param in dist_params]
+    new_dist_params = [
+        at.broadcast_to(
+            param,
+            at.broadcast_shape(
+                tuple(param.shape), tuple(bcast_shape), arrays_are_shapes=True
+            ),
+        )
+        for param in dist_params
+    ]
     bcasted_node = lifted_node.op.make_node(rng, size, dtype, *new_dist_params)
 
     if aesara.config.compute_test_value != "off":
@@ -178,8 +162,11 @@ def naive_bcast_rv_lift(fgraph, node):
     return [bcasted_node.outputs[1]]
 
 
-logprob_canonicalize = SequenceDB()
-
-
-logprob_canonicalize.register("canonicalize", optdb["canonicalize"], -10, "basic")
-logprob_canonicalize.register("rvsinker", RVSinker(), -1, "basic")
+logprob_rewrites_db = SequenceDB()
+logprob_rewrites_db.register("canonicalize", optdb["canonicalize"], -10, "basic")
+rv_sinking_db = EquilibriumDB()
+rv_sinking_db.register("dimshuffle_lift", local_dimshuffle_rv_lift, -5, "basic")
+rv_sinking_db.register("subtensor_lift", local_subtensor_rv_lift, -5, "basic")
+rv_sinking_db.register("broadcast_to_lift", naive_bcast_rv_lift, -5, "basic")
+rv_sinking_db.register("incsubtensor_lift", incsubtensor_rv_replace, -5, "basic")
+logprob_rewrites_db.register("sinking", rv_sinking_db, -10, "basic")
