@@ -17,12 +17,15 @@ from aesara import config
 from aesara.graph.basic import Constant, Variable, clone_get_equiv, graph_inputs, walk
 from aesara.graph.fg import FunctionGraph
 from aesara.graph.op import compute_test_value
+from aesara.graph.opt_utils import optimize_graph
 from aesara.graph.type import CType
-from aesara.tensor.random.op import RandomVariable
+from aesara.tensor.basic_opt import ShapeFeature, topo_constant_folding
 from aesara.tensor.sharedvar import SharedVariable
 from aesara.tensor.subtensor import AdvancedIncSubtensor, AdvancedIncSubtensor1
 from aesara.tensor.var import TensorVariable
 
+from aeppl.abstract import MeasurableVariable
+
 PotentialShapeType = Union[
     int,
     np.ndarray,
@@ -32,11 +35,6 @@ PotentialShapeType = Union[
 ]
 
 
-def logsumexp(x, axis=None, keepdims=True):
-    res = at.log(at.sum(at.exp(x), axis=axis, keepdims=True))
-    return res if keepdims else res.squeeze()
-
-
 def change_rv_size(
     rv_var: TensorVariable,
     new_size: PotentialShapeType,
@@ -88,18 +86,18 @@ def extract_rv_and_value_vars(
     Parameters
     ==========
     var
-        A variable corresponding to a ``RandomVariable``.
+        A variable corresponding to a ``MeasurableVariable``.
 
     Returns
     =======
-    The first value in the tuple is the ``RandomVariable`` and the second is
+    The first value in the tuple is the ``MeasurableVariable`` and the second is
     the observations variable or the measure/log-probability value variable.
 
     """
     if not var.owner:
         return None, None
 
-    if isinstance(var.owner.op, RandomVariable):
+    if isinstance(var.owner.op, MeasurableVariable):
         rv_value = getattr(var.tag, "observations", getattr(var.tag, "value_var", None))
         return var, rv_value
 
@@ -134,18 +132,18 @@ def walk_model(
     graphs: Iterable[TensorVariable],
     walk_past_rvs: bool = False,
     stop_at_vars: Optional[Set[TensorVariable]] = None,
-    expand_fn: Callable[[TensorVariable], Iterable[TensorVariable]] = lambda var: [],
+    expand_fn: Callable[[TensorVariable], List[TensorVariable]] = lambda var: [],
 ) -> Generator[TensorVariable, None, None]:
     """Walk model graphs and yield their nodes.
 
-    By default, these walks will not go past ``RandomVariable`` nodes.
+    By default, these walks will not go past ``MeasurableVariable`` nodes.
 
     Parameters
     ==========
     graphs
         The graphs to walk.
     walk_past_rvs
-        If ``True``, the walk will not terminate at ``RandomVariable``s.
+        If ``True``, the walk will not terminate at ``MeasurableVariable``s.
     stop_at_vars
         A list of variables at which the walk will terminate.
     expand_fn
@@ -154,12 +152,12 @@ def walk_model(
     if stop_at_vars is None:
         stop_at_vars = set()
 
-    def expand(var):
+    def expand(var: TensorVariable, stop_at_vars=stop_at_vars) -> List[TensorVariable]:
         new_vars = expand_fn(var)
 
         if (
             var.owner
-            and (walk_past_rvs or not isinstance(var.owner.op, RandomVariable))
+            and (walk_past_rvs or not isinstance(var.owner.op, MeasurableVariable))
             and (var not in stop_at_vars)
         ):
             new_vars.extend(reversed(var.owner.inputs))
@@ -171,7 +169,10 @@ def walk_model(
 
 def replace_rvs_in_graphs(
     graphs: Iterable[TensorVariable],
-    replacement_fn: Callable[[TensorVariable], Dict[TensorVariable, TensorVariable]],
+    replacement_fn: Callable[
+        [TensorVariable, Dict[TensorVariable, TensorVariable]],
+        Dict[TensorVariable, TensorVariable],
+    ],
     initial_replacements: Optional[Dict[TensorVariable, TensorVariable]] = None,
     **kwargs,
 ) -> Tuple[TensorVariable, Dict[TensorVariable, TensorVariable]]:
@@ -193,9 +194,9 @@ def replace_rvs_in_graphs(
     if initial_replacements:
         replacements.update(initial_replacements)
 
-    def expand_replace(var):
-        new_nodes = []
-        if var.owner and isinstance(var.owner.op, RandomVariable):
+    def expand_replace(var: TensorVariable) -> List[TensorVariable]:
+        new_nodes: List[TensorVariable] = []
+        if var.owner and isinstance(var.owner.op, MeasurableVariable):
             new_nodes.extend(replacement_fn(var, replacements))
         return new_nodes
 
@@ -281,3 +282,16 @@ def indices_from_subtensor(idx_list, indices):
         if idx_list
         else indices
     )
+
+
+def get_constant_value(x):
+    """Use constant folding to get a constant value."""
+
+    axis_fg = FunctionGraph(outputs=[x], features=[ShapeFeature()], clone=True)
+
+    (folded_axis,) = optimize_graph(axis_fg, custom_opt=topo_constant_folding).outputs
+
+    if not isinstance(folded_axis, Constant):
+        raise ValueError("Could not obtain a constant from constant folding")
+
+    return folded_axis.data
